/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

let {Prefs} = require("prefs");
let {WindowObserver} = require("windowObserver");
let {getSchemeCorrection, isKnownScheme, getDomainCorrection, getDomainReferral, onWhitelistEntryRemoved} = require("rules");
let {processTypedDomain, processDomainCorrection,
		 processUserCorrection, processFalsePositive} = require("typedItCollector");
let appIntegration = require("appIntegration");

// Attach our handlers to all browser windows
new WindowObserver(
{
	applyToWindow: function(window)
	{
		if (!appIntegration.isKnownWindow(window))
			return;

		let browser = appIntegration.getBrowser(window);
		if (browser)
			browser.addEventListener("DOMContentLoaded", handlePageLoad, false);

		appIntegration.applyToWindow(window, correctURL);
	},

	removeFromWindow: function(window)
	{
		if (!appIntegration.isKnownWindow(window))
			return;

		let browser = appIntegration.getBrowser(window);
		if (browser)
		{
			browser.removeEventListener("DOMContentLoaded", handlePageLoad, false);
			if (browser.browsers)
			{
				for (let i = 0; i < browser.browsers.length; i++)
				{
					let contentWnd = browser.browsers[i].contentWindow;
					if (contentWnd.document.documentURI.indexOf("about:neterror?") == 0)
						removeFromNetErrorPage(contentWnd);
				}
			}
		}

		appIntegration.removeFromWindow(window);
	}
});

// Load HTML code to add to network error pages
let netErrorOverlay = null;
let request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
request.open("GET", "chrome://url-fixer/content/netError.xhtml");
request.addEventListener("load", function(event)
{
	netErrorOverlay = event.target.responseXML;
}, false);
request.send(null)

function handlePageLoad(event)
{
	let document = event.target;
	if (document.documentURI.indexOf("about:neterror?") != 0 ||
			document.documentURI.indexOf("e=netOffline") > 0 ||
			document.documentURI.indexOf("e=notCached") > 0)
	{
		return;
	}

	if (!netErrorOverlay || document.getElementById("url-fixer-section"))
		return;

	let container = document.getElementById("errorPageContainer");
	if (!container)
		return;

	container.appendChild(netErrorOverlay.documentElement.cloneNode(true));

	let textField = document.getElementById("url-fixer-intention");
	textField.value = document.defaultView.location.href;

	let retryButton = document.getElementById("url-fixer-retry");
	retryButton.addEventListener("click", function()
	{
		let newURL = textField.value.replace(/^\s+/, "").replace(/\s+$/, "");
		if (!newURL.length)
			return;

		let [prefix, newHost, suffix] = parseURL(newURL);
		let oldHost = document.defaultView.location.hostname.toLowerCase();

		processUserCorrection(oldHost, newHost);

		if (newHost.indexOf("www.") == 0 && oldHost.indexOf("www.") == 0)
		{
			// Ignore www. prefix if they both start with it
			newHost = newHost.substr(4);
			oldHost = oldHost.substr(4);
		}
		if (oldHost && newHost != oldHost)
		{
			Prefs.custom_replace[oldHost] = newHost;
			Prefs.custom_replace = JSON.parse(JSON.stringify(Prefs.custom_replace));
		}
		
		// Remove from whitelist
		if (oldHost in Prefs.whitelist)
		{
			delete Prefs.whitelist[oldHost];
			onWhitelistEntryRemoved(oldHost);
			Prefs.whitelist = JSON.parse(JSON.stringify(Prefs.whitelist));
		}
		
		document.defaultView.location.replace(newURL);
	}, false);
}

function removeFromNetErrorPage(window)
{
	let overlay = window.document.getElementById("url-fixer-section");
	if (overlay)
		overlay.parentNode.removeChild(overlay);
}

function parseURL(url)
{
	if (/^\s*((?:\w+:)?\/*(?:[^\/#]*@)?)([^\/:#]*)/.test(url))
		return [RegExp.$1, RegExp.$2.toLowerCase(), RegExp.rightContext];
	else
		return [url, null, null];
}

function isIPAddress(domain)
{
	try
	{
		Services.eTLD.getBaseDomainFromHost(domain);
		return false;
	}
	catch (e)
	{
		return (e.result == Cr.NS_ERROR_HOST_IS_IP_ADDRESS);
	}
}

function correctURL(window, value)
{
	let hasCorrection = false;

	value = value.trim();
	if (value.length == 0)
		return null;

	// Replace backslashes
	value = value.replace(/\\/g, "/");

	// Does the URL scheme need correcting?
	if (/^([^\/]+)(\/.*)/.test(value))
	{
		let scheme = RegExp.$1;
		let suffix = RegExp.$2;
		let correction = getSchemeCorrection(scheme)
		if (correction != scheme)
		{
			value = correction + suffix;
			hasCorrection = true;
		}
	}

	// Ignore URL schemes that we don't know
	if (/^([\w\-]+:)/.test(value) && !isKnownScheme(RegExp.$1))
		return null;

	// Ignore search keywords and such
	if ("getShortcutOrURI" in window && window.getShortcutOrURI(value) != value)
		return null;

	// Spaces before the first slash or period is probably a quick search
	if (/^[^\/\.\s]+\s/.test(value))
		return null;

	let [prefix, domain, suffix] = parseURL(value);
	if (!domain)
		return null;

	let oldDomain = domain;
	if (!isIPAddress(domain))
	{
		processTypedDomain(domain);

		let newDomain = getDomainCorrection(domain);
		if (newDomain != domain)
		{
			processDomainCorrection(domain, newDomain);
			domain = newDomain;
			hasCorrection = true;

			let referral = getDomainReferral(domain.replace(/^www\./, ""));
			if (referral)
			{
				// We need to add a query string parameter when sending users to this domain
				let anchorIndex = suffix.indexOf("#");
				let anchor = "";
				if (anchorIndex >= 0)
				{
					anchor = suffix.substr(anchorIndex);
					suffix = suffix.substr(0, anchorIndex);
				}

				let queryIndex = suffix.indexOf("?");
				if (queryIndex >= 0)
				{
					if (!/&$/.test(suffix))
						suffix += "&";
					suffix += referral;
				}
				else
				{
					if (suffix.indexOf("/") < 0)
						suffix += "/";
					suffix += "?" + referral;
				}

				suffix += anchor;
			}
		}
	}

	if (!hasCorrection)
		return null;

	// Show infobar to inform and ask about correction
	let [message, yes, no] = getInfobarTexts();
	message = message.replace(/\?1\?/g, prefix+domain);
	let buttons = [
		{
			label:      yes,
			accessKey:  null,
			callback:   function()
			{
				// Yes: Do nothing
			}
		},
		{
			label:      no,
			accessKey:  null,
			callback:   function()
			{
				// No: Add to list of corrections (ignore)
				let {onWhitelistEntryAdded} = require("rules");
				let entry = oldDomain.replace(/^www\./, "");
				Prefs.whitelist[entry] = true;
				onWhitelistEntryAdded(entry);
				Prefs.whitelist = JSON.parse(JSON.stringify(Prefs.whitelist));

				require("appIntegration").loadURI(window, value);
				processFalsePositive(domain, oldDomain);
			}
		}
	];
	// We need to have persistence being set to 1 due to redirect which happens afterwards
	require("appIntegration").openInfobar(window, "url-fixer-infobar-askafter", message, buttons, 1);

	require("survey").incrementCorrectionsCounter();

	return prefix + domain + suffix;
}

let stringBundle = null;

function getInfobarTexts()
{
	// Randomize URI to work around bug 719376
	if (!stringBundle)
		stringBundle = Services.strings.createBundle("chrome://url-fixer/locale/locale.properties?" + Math.random());
	let result = [
		stringBundle.GetStringFromName("urlfixer.isItCorrect"),
		stringBundle.GetStringFromName("urlfixer.yes"),
		stringBundle.GetStringFromName("urlfixer.no")
	];

	getInfobarTexts = function() result;
	return getInfobarTexts();
}
